<?php
/**
 * This file is part of WooCommerce Birthdate Field.
 *
 * WooCommerce Birthdate Field is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 2 of the License, or any later version.
 *
 * WooCommerce Birthdate Field is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * WooCommerce Birthdate Field.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Plugin Name:       WooCommerce Birthdate Field
 * Plugin URI:        https://extensions.tandem.blog/woocommerce-birthdate
 * Description:       Adds a birthdate field to the billing address.
 * Version:           2023061600
 * Author:            Tàndem
 * Author URI:        https://tandem.blog/
 * Text Domain:       woocommerce-birthdate
 * Domain Path:       /languages
 * Requires PHP:      7.4
 * Requires at least: 5.5.0
 * License:           GPL-2.0-or-later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 */

declare( strict_types=1 );

if ( ! class_exists( 'Woocommerce_Birthdate' ) ) :

	class Woocommerce_Birthdate {
		private const BILLING_ADDRESS = 'billing';
		private const BIRTHDATE_FIELD = 'birthdate';
		private const BILLING_BIRTHDATE_FIELD = self::BILLING_ADDRESS . '_' . self::BIRTHDATE_FIELD;
		private const BILLING_BIRTHDATE_META = '_' . self::BILLING_BIRTHDATE_FIELD;
		private static ?Woocommerce_Birthdate $instance = null;

		private function __construct() {
			add_action( 'plugins_loaded', [ $this, 'setup_translation' ] );

			add_filter( 'woocommerce_checkout_fields', [ $this, 'add_frontend_birthdate_field' ] );
			add_filter( 'woocommerce_admin_billing_fields', [ $this, 'add_admin_birthdate_field' ] );
			add_action( 'woocommerce_after_checkout_validation', [ $this, 'validate_birthdate' ], 10, 2 );

			add_action( 'woocommerce_checkout_create_order', [ $this, 'add_birthdate_meta_from_checkout' ], 10, 2 );
			add_filter(
				'woocommerce_rest_pre_insert_shop_order_object',
				[ $this, 'add_birthdate_meta_from_rest' ],
				10,
				2
			);
		}

		public static function getInstance(): self {
			if ( ! self::$instance ) {
				self::$instance = new self();
			}

			return self::$instance;
		}

		public function setup_translation(): void {
			load_plugin_textdomain(
				'woocommerce-birthdate',
				false,
				dirname( plugin_basename( __FILE__ ) ) . '/languages'
			);
		}

		public function add_frontend_birthdate_field( array $fields ): array {
			$fields[ self::BILLING_ADDRESS ][ self::BILLING_BIRTHDATE_FIELD ] = [
				'label' => __( 'Birthdate', 'woocommerce-birthdate' ),
				'required' => true,
				'type' => 'date',
				'class' => [ 'form-row-wide' ],
				'autocomplete' => 'bday',
				'priority' => 25,
			];

			return $fields;
		}

		public function add_admin_birthdate_field( array $fields ): array {
			$fields[ self::BIRTHDATE_FIELD ] = [
				'label' => __( 'Birthdate', 'woocommerce-birthdate' ),
				'show' => true,
			];

			return $fields;
		}

		public function validate_birthdate( array $data, WP_Error $errors ): void {
			$date_input = $data[ self::BILLING_BIRTHDATE_FIELD ] ?? '';
			if ( empty( $date_input ) ) {
				return;
			}

			$date = DateTimeImmutable::createFromFormat( 'Y-m-d', $date_input );
			if ( $date === false ) {
				// translators: %s is the raw text from the input field.
				$errors->add( self::BILLING_BIRTHDATE_FIELD . '_validation', sprintf( __( '%s is not a valid date format', 'woocommerce-birthdate' ), $date_input ), [ 'id' => self::BILLING_BIRTHDATE_FIELD ] );

				return;
			}
			$today = new DateTimeImmutable( 'today' );
			$earliest = $today->setDate( $today->format( 'Y' ) - 130, 1, 1 );
			if ( $date < $earliest || $date >= $today ) {
				$format = __( 'm/d/Y', 'woocommerce-birthdate' );
				// translators: %s is the date already formatted.
				$errors->add( self::BILLING_BIRTHDATE_FIELD . '_validation', sprintf( __( '%s is not a valid birthdate', 'woocommerce-birthdate' ), $date->format( $format ) ), [ 'id' => self::BILLING_BIRTHDATE_FIELD ] );
			}
		}

		public function add_birthdate_meta_from_checkout( WC_Order $order, array $data ): void {
			if ( isset( $data[ self::BILLING_BIRTHDATE_FIELD ] ) ) {
				$this->add_billing_birthdate( $order, $data[ self::BILLING_BIRTHDATE_FIELD ] );
			}
		}

		public function add_birthdate_meta_from_rest( WC_Order $order, $request ): WC_Order {
			$billing = $request[ self::BILLING_ADDRESS ] ?? [];
			if ( ! empty( $billing ) && array_key_exists( self::BIRTHDATE_FIELD, $billing ) ) {
				$this->add_billing_birthdate( $order, $billing[ self::BIRTHDATE_FIELD ] );
			}

			return $order;
		}

		private function add_billing_birthdate( WC_Order $order, string $birthdate ): void {
			$order->add_meta_data( self::BILLING_BIRTHDATE_META, $birthdate, true );
		}

	}

	Woocommerce_Birthdate::getInstance();

endif;
